@@ -1,6 +1,7 @@
 
 import os
 import sys
+import fies
 import shutil
 from sout import sout
 from pathlib import Path
@@ -9,6 +10,66 @@ from .parts.config import develop_header
 # developバージョンのツールを読み込み [ezpip]
 from .parts.load_develop import load_develop
 
+# version_historyのファイルをロード
+def load_version_history(filename):
+	# 存在しない場合ファイルを作る
+	if os.path.exists(filename) is False:
+		fies[filename, "json"] = []
+	# 読み込み
+	version_history = fies[filename]
+	return version_history
+
+# ワイルドカード指定に該当するものの一覧を取得
+def wild_search(data_ls, query):
+	head, tail = query.split("*")
+	wild_ls = []
+	for e in data_ls:
+		if len(e) < len(query)-1: continue
+		if e.startswith(head) is False: continue
+		if e.endswith(tail) is False: continue
+		wild_ls.append(e[len(head):len(e)-len(tail)])
+	return wild_ls
+
+# 渡された処理がエラーを起こすかを判断
+def judge_error(func):
+	try:
+		func()
+		return False
+	except:
+		return True
+
+# ワイルドカード指定に該当する「次の」のインデックスを取得
+def get_new_idx(version_history, version):
+	# ワイルドカード指定に該当するものの一覧を取得
+	wild_ls = wild_search(version_history, version)
+	# int化 (int化できないものは飛ばす)
+	int_wild_ls = [int(e) for e in wild_ls
+		if judge_error(lambda: int(e)) is False]
+	# 履歴なき場合は0を返す
+	if len(int_wild_ls) == 0: return 0
+	# 最大のidxの次のidxを返す
+	new_idx = max(int_wild_ls) + 1
+	return new_idx
+
+# 最新versionを自動的に決める
+def gen_latest_version(version):
+	filename = "__ezpip_version_history__.json"
+	# verison未指定の場合
+	if version is None: return None
+	# ワイルドカード指定あり / なし 分岐
+	if "*" in version:
+		# ワイルドカードが「ちょうど1つ」かを確認
+		if len(version.split("*"))-1 != 1: raise Exception("[ezpip error] ワイルドカード「*」の指定はちょうど1つである必要があります。")
+		# ワイルドカード指定がある場合
+		version_history = load_version_history(filename)	# version_historyのファイルをロード
+		new_idx = get_new_idx(version_history, version)	# ワイルドカード指定に該当する「次の」のインデックスを取得
+		new_version = version.replace("*", str(new_idx))
+	else:
+		# ワイルドカード指定がない場合
+		new_version = version
+	fies[filename] = fies[filename] + [new_version]	# 追記
+	return new_version
+
 # パスの末尾部分を分離
 def div_path_tail(arg_path):
 	# しっぽの取得
@@ -88,11 +149,12 @@ def get_long_description(md_path):
 # パッケージ情報にアクセスするインターフェース (with構文内)
 class _EzpipPackageResult:
 	# 初期化処理
-	def __init__(self, cleanup_proc, publish_name, packages, root_dir):
+	def __init__(self, cleanup_proc, publish_name, packages, root_dir, version):
 		self.cleanup_proc = cleanup_proc
 		self.publish_name = publish_name
 		self._packages = packages
 		self._root_dir = root_dir
+		self._version = version
 	# with突入時の処理 (with構文のas節内に代入される)
 	def __enter__(self):
 		return self
@@ -106,6 +168,11 @@ class _EzpipPackageResult:
 		elif attr_name == "long_description":
 			# long_description(README.md)の取得
 			return get_long_description(md_path = self._root_dir + "/README.md")
+		elif attr_name == "version":
+			if self._version is None: raise Exception("[ezpip error] version が指定されていません。")
+			return self._version
+		else:
+			raise Exception("[ezpip error] AttributeError: this object has no attribute '%s'"%attr_name)
 	# 文字列化(その1)
 	def __str__(self):
 		return "<_EzpipPackageResult package_name=%s>"%self.publish_name
@@ -114,7 +181,9 @@ class _EzpipPackageResult:
 		return str(self)
 
 # 公開用パッケージの作成 [ezpip]
-def packager(develop_dir):
+def packager(develop_dir, version = None):
+	# 最新versionを自動的に決める
+	new_version = gen_latest_version(version)
 	# develop_dirの名前の規則と存在の確認 (問題がある場合は落とす)
 	check_develop_dir(develop_dir)
 	# 公開用パッケージ名を生成
@@ -130,4 +199,4 @@ def packager(develop_dir):
 		del_dir(publish_dir)
 	# packagesの列挙
 	packages = listup_packages(publish_dir)
-	return _EzpipPackageResult(cleanup_proc, publish_name, packages, root_dir = publish_dir + "/../")
+	return _EzpipPackageResult(cleanup_proc, publish_name, packages, root_dir = publish_dir + "/../", version = new_version)
